/****************************************************************************/
/*  File:       TextResponseBody.java                                       */
/*  Author:     F. Georges - fgeorges.org                                   */
/*  Date:       2009-02-02                                                  */
/*  Tags:                                                                   */
/*      Copyright (c) 2009 Florent Georges (see end of file.)               */
/* ------------------------------------------------------------------------ */


package org.expath.httpclient.impl;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.UnsupportedEncodingException;
import org.expath.httpclient.TreeBuilderHelper;
import java.util.List;
import net.sf.saxon.om.Item;
import net.sf.saxon.trans.XPathException;
import net.sf.saxon.value.StringValue;
import org.apache.http.Header;
import org.expath.httpclient.ContentType;
import org.expath.httpclient.HeaderSet;
import org.expath.httpclient.HttpResponse;
import org.expath.httpclient.HttpResponseBody;

/**
 * TODO<doc>: ...
 *
 * @author Florent Georges
 * @date   2009-02-02
 */
public class TextResponseBody
        implements HttpResponseBody
{
//    public TextResponseBody(String value, ContentType type, HeaderSet headers)
//            throws XPathException
//    {
//        myContent = value;
//        myContentType = type;
//        // TODO: ...
//        myEncoding = null;
//        myHeaders = headers;
//        myResponse = null;
//    }

    public TextResponseBody(InputStream in, ContentType type, HeaderSet headers)
            throws XPathException
    {
        // FIXME: ...
        String charset = "utf-8";
        try {
            Reader reader = new InputStreamReader(in, charset);
            init(reader, type, headers);
        }
        catch ( UnsupportedEncodingException ex ) {
            String msg = "not supported charset reading HTTP response: " + charset;
            throw new XPathException(msg, ex);
        }
    }

    public TextResponseBody(Reader in, ContentType type, HeaderSet headers)
            throws XPathException
    {
        init(in, type, headers);
    }

    private void init(Reader in, ContentType type, HeaderSet headers)
            throws XPathException
    {
        myContentType = type;
        myHeaders = headers;
        myResponse = null;
        // BufferedReader handles the ends of line (all \n, \r, and \r\n are
        // transformed to \n)
        try {
            StringBuilder builder = new StringBuilder();
            BufferedReader buf_in = new BufferedReader(in);
            String buf = null;
            while ( (buf = buf_in.readLine()) != null ) {
                builder.append(buf);
                builder.append('\n');
            }
            myContent = builder.toString();
        }
        catch ( IOException ex ) {
            throw new XPathException("error reading HTTP response", ex);
        }
    }

    public void setResponse(HttpResponse response)
    {
        myResponse = response;
    }

    public void outputBody(TreeBuilderHelper b)
            throws XPathException
    {
        if ( myHeaders != null ) {
            b.outputHeaders(myHeaders);
        }
        b.startElem("body");
        b.attribute("media-type", myContentType.getValue());
        // TODO: Support other attributes as well?
        b.startContent();
        b.endElem();
    }

    public void addResultItems(List<Item> sequence)
            throws XPathException
    {
        sequence.add(new StringValue(myContent));
    }

    private String getHeader(String name)
            throws XPathException
    {
        HeaderSet headers = myHeaders;
        if ( headers == null ) {
            headers = myResponse.getHeaders();
        }
        Header h = headers.getFirstHeader(name);
        return h == null ? null : h.getValue();
    }

    private String myContent;
    private ContentType myContentType;
    private HeaderSet myHeaders;
    private HttpResponse myResponse;
}


/* ------------------------------------------------------------------------ */
/*  DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS COMMENT.               */
/*                                                                          */
/*  The contents of this file are subject to the Mozilla Public License     */
/*  Version 1.0 (the "License"); you may not use this file except in        */
/*  compliance with the License. You may obtain a copy of the License at    */
/*  http://www.mozilla.org/MPL/.                                            */
/*                                                                          */
/*  Software distributed under the License is distributed on an "AS IS"     */
/*  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See    */
/*  the License for the specific language governing rights and limitations  */
/*  under the License.                                                      */
/*                                                                          */
/*  The Original Code is: all this file.                                    */
/*                                                                          */
/*  The Initial Developer of the Original Code is Florent Georges.          */
/*                                                                          */
/*  Contributor(s): none.                                                   */
/* ------------------------------------------------------------------------ */
